package sun.awt;

import java.lang.reflect.Array;
import java.util.EventListener;

public class EventListenerAggregate
{
    private EventListener[] listenerList;
    
    public EventListenerAggregate(Class<? extends EventListener> componentType) {
        if (componentType == null) {
            throw new NullPointerException("listener class is null");
        }
        this.listenerList = (EventListener[])(EventListener[])Array.newInstance(componentType, 0);
        return;
    }
    
    private Class<?> getListenerClass() {
        return this.listenerList.getClass().getComponentType();
    }
    
    public synchronized void add(EventListener obj) {
        Class<Object> listenerClass = this.getListenerClass();
        if (!listenerClass.isInstance(obj)) {
            throw new ClassCastException("listener " + obj + " is not an instance of listener class " + listenerClass);
        }
        EventListener[] listenerList = (EventListener[])(EventListener[])Array.newInstance(listenerClass, this.listenerList.length + 1);
        System.arraycopy(this.listenerList, 0, listenerList, 0, this.listenerList.length);
        listenerList[this.listenerList.length] = obj;
        this.listenerList = listenerList;
        return;
    }
    
    public synchronized boolean remove(EventListener eventListener) {
        Class<Object> listenerClass = this.getListenerClass();
        if (!listenerClass.isInstance(eventListener)) {
            throw new ClassCastException("listener " + eventListener + " is not an instance of listener class " + listenerClass);
        }
        int i = 0;
        while (i < this.listenerList.length) {
            if (this.listenerList[i].equals(eventListener)) {
                EventListener[] listenerList = (EventListener[])(EventListener[])Array.newInstance(listenerClass, this.listenerList.length - 1);
                System.arraycopy(this.listenerList, 0, listenerList, 0, i);
                System.arraycopy(this.listenerList, i + 1, listenerList, i, this.listenerList.length - i - 1);
                this.listenerList = listenerList;
                return true;
            }
            ++i;
        }
        return false;
    }
    
    public synchronized EventListener[] getListenersInternal() {
        return this.listenerList;
    }
    
    public synchronized EventListener[] getListenersCopy() {
        return (this.listenerList.length == 0) ? this.listenerList : ((EventListener[])this.listenerList.clone());
    }
    
    public synchronized int size() {
        return this.listenerList.length;
    }
    
    public synchronized boolean isEmpty() {
        return this.listenerList.length == 0;
    }
}
